@@ -1,7 +1,7 @@
 package Path::Router;
 use Moose;
 
-our $VERSION   = '0.06';
+our $VERSION   = '0.07';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use File::Spec::Unix ();
@@ -10,7 +10,7 @@ use Path::Router::Types;
 use Path::Router::Route;
 use Path::Router::Route::Match;
 
-our $DEBUG = 0;
+use constant DEBUG => exists $ENV{PATH_ROUTER_DEBUG} ? $ENV{PATH_ROUTER_DEBUG} : 0;
 
 has 'routes' => (
     is      => 'ro', 
@@ -18,97 +18,96 @@ has 'routes' => (
     default => sub { [] },
 );
 
+has 'inline' => (
+    is      => 'rw',
+    isa     => 'Bool',
+    default => 1,
+    trigger => sub { $_[0]->clear_match_code }
+);
+
+has 'match_code' => (
+    is         => 'rw',
+    isa        => 'CodeRef',
+    lazy_build => 1,
+    clearer    => 'clear_match_code'
+);
+
+sub _build_match_code {
+    my $self = shift;
+
+    my @code;
+    my $i = 0;
+    foreach my $route (@{$self->routes}) {
+        push @code, $route->generate_match_code($i++);
+    }
+
+    my $code = "sub {\n" .
+        "#line " . __LINE__ . ' "' . __FILE__ . "\"\n" .
+        "   my \$self = shift;\n" .
+        "   my \$path = shift;\n" .
+        "   my \$routes = \$self->routes;\n" .
+        join("\n", @code) .
+        "#line " . __LINE__ . ' "' . __FILE__ . "\"\n" .
+        "   print STDERR \"match failed\\n\" if DEBUG();\n" .
+        "   return ();\n" .
+        "}"
+    ;
+    # print STDERR $code;
+    eval $code or warn $@;
+}
+
 sub add_route {
     my ($self, $path, %options) = @_;
     push @{$self->routes} => Path::Router::Route->new(
         path  => $path, 
         %options
     );
+    $self->clear_match_code;
+}
+
+sub insert_route {
+    my ($self, $path, %options) = @_;
+    my $at = delete $options{at} || 0;
+
+    my $route = Path::Router::Route->new(
+        path  => $path, 
+        %options
+    );
+    my $routes = $self->routes;
+
+    if (! $at) {
+        unshift @$routes, $route;
+    } elsif ($#{$routes} < $at) {
+        push @$routes, $route;
+    } else {
+        splice @$routes, $at, 0, $route;
+    }
+    $self->clear_match_code;
 }
 
 sub match {
     my ($self, $url) = @_;
+
+    if ($self->inline) {
+        $url =~ s|/{2,}|/|g;                          # xx////xx  -> xx/xx
+        $url =~ s{(?:/\.)+(?:/|\z)}{/}g;              # xx/././xx -> xx/xx
+        $url =~ s|^(?:\./)+||s unless $url eq "./";   # ./xx      -> xx
+        $url =~ s|^/(?:\.\./)+|/|;                    # /../../xx -> xx
+        $url =~ s|^/\.\.$|/|;                         # /..       -> /
+        $url =~ s|/\z|| unless $url eq "/";           # xx/       -> xx
+        $url =~ s|^/||; # Path::Router specific. remove first /
+
+        return $self->match_code->($self, $url);
+    } else {
+        my @parts = grep { defined $_ and length $_ }
+            split '/' => File::Spec::Unix->canonpath($url);
     
-    my @parts = grep { $_ } split '/' => File::Spec::Unix->canonpath($url);
-    
-    foreach my $route (@{$self->routes}) {
-        my $mapping;
-        
-        eval {           
-            
-            warn "> Attempting to match ", $route->path, " to (", (join " / " => @parts), ")" if $DEBUG;
-            
-            if ($DEBUG) {
-                warn "parts length: " . scalar @parts;
-                warn "route length: " . $route->length;                
-                warn "route length w/out optionals: " . $route->length_without_optionals;
-                warn join ", " => @{$route->components};
-            }
-            
-            # they must be the same length
-            (
-                scalar(@parts) >= $route->length_without_optionals &&
-                scalar(@parts) <= $route->length 
-            ) || die "LENGTHS DID NOT MATCH\n";
-                
-            warn "\t... They are the same length" if $DEBUG;
-        
-            my @components = @{$route->components};
-            
-            if ($route->has_defaults) {
-                warn "\t... ", $route->path, " has a guide" if $DEBUG;
-                $mapping = $route->create_default_mapping;
-            }
-        
-            foreach my $i (0 .. $#components) {
-                
-                if (!defined $parts[$i] && $route->is_component_optional($components[$i])) {
-                    next;
-                }
-                
-                # if it is a variable (starts with a colon)
-                if ($route->is_component_variable($components[$i])) {
-                    my $name = $route->get_component_name($components[$i]);
-                    
-                    warn "\t\t... mapped ", $components[$i], " to ", $parts[$i] if $DEBUG;
-                    
-                    if (my $type = $route->has_validation_for($name)) {
-                        
-                        warn "\t\t\t... checking validation for $name against ", $type->name ," and ", $parts[$i] if $DEBUG;                            
-                        
-                        $type->check($parts[$i]) || die "VALIDATION DID NOT PASS\n";
-                        
-                        warn "\t\t\t\t... validation passed for $name with ", $parts[$i] if $DEBUG;
-                    }
-                    
-                    $mapping->{$name} = $parts[$i];
-                }
-                else {
-                    warn "\t\t... found a constant (", $components[$i], ")" if $DEBUG;
-                    
-                    ($components[$i] eq $parts[$i]) || die "CONSTANT DID NOT MATCH\n";
-                    
-                    warn "\t\t\t... constant matched" if $DEBUG;
-                }
-            }
-        
-        };
-        unless ($@) {
-            warn "+ ", $route->path, " matched ", $url if $DEBUG;
-            
-            return Path::Router::Route::Match->new(
-                path    => (join "/" => @parts),
-                route   => $route,
-                mapping => $mapping || {},
-            );
+        for my $route (@{$self->routes}) {
+            my $match = $route->match(\@parts) or next;
+            return $match;
         }
-        else {
-            warn "~ got an exception here : ", $@ if $DEBUG;
-            warn "\t- ", $route->path, " did not match ", $url, " because ", $@ if $DEBUG;
-        }
-        
     }
-    
+    return;
 }
 
 sub uri_for {
@@ -149,7 +148,7 @@ sub uri_for {
 
             my @keys = keys %url_map;
 
-            if ($DEBUG) {
+            if (DEBUG) {
                 warn "> Attempting to match ", $route->path, " to (", (join " / " => @keys), ")";
             }
             (
@@ -158,27 +157,27 @@ sub uri_for {
             ) || die "LENGTH DID NOT MATCH\n";
 
             if (my @missing = grep { ! exists $url_map{$_} } keys %required) {
-                warn "missing: @missing" if $DEBUG;
+                warn "missing: @missing" if DEBUG;
                 die "MISSING ITEM [@missing]\n";
             }
 
             if (my @extra = grep {
                     ! $required{$_} && ! $optional{$_} && ! $match{$_}
                 } keys %url_map) {
-                warn "extra: @extra" if $DEBUG;
+                warn "extra: @extra" if DEBUG;
                 die "EXTRA ITEM [@extra]\n";
             }
 
             if (my @nomatch = grep {
                     exists $url_map{$_} and $url_map{$_} ne $match{$_}
                 } keys %match) {
-                warn "no match: @nomatch" if $DEBUG;
+                warn "no match: @nomatch" if DEBUG;
                 die "NO MATCH [@nomatch]\n";
             }
 
             for my $component (@{$route->components}) {
                 if ($route->is_component_variable($component)) {
-                    warn "\t\t... found a variable ($component)" if $DEBUG;
+                    warn "\t\t... found a variable ($component)" if DEBUG;
                     my $name = $route->get_component_name($component);
                     
                     push @url => $url_map{$name}
@@ -190,12 +189,12 @@ sub uri_for {
                 }
 
                 else {
-                    warn "\t\t... found a constant ($component)" if $DEBUG;
+                    warn "\t\t... found a constant ($component)" if DEBUG;
                     
                     push @url => $component;
                 }                    
                 
-                warn "+++ URL so far ... ", (join "/" => @url) if $DEBUG;
+                warn "+++ URL so far ... ", (join "/" => @url) if DEBUG;
             }
             
         };
@@ -206,7 +205,7 @@ sub uri_for {
             do {
                 warn join "/" => @url;
                 warn "... ", $@;
-            } if $DEBUG;
+            } if DEBUG;
         }
         
     }
@@ -331,6 +330,33 @@ It is in Perl :)
 
 =item B<add_route ($path, ?%options)>
 
+Adds a new route to the I<end> of the routes list.
+
+=item B<insert_route ($path, %options)>
+
+Adds a new route to the routes list. You may specify an C<at> parameter, which would 
+indicate the position where you want to insert your newly created route. The C<at> 
+parameter is the C<index> position in the list, so it starts at 0.
+
+Examples:
+
+    # You have more than three paths, insert a new route at
+    # the 4th item 
+    $router->insert_route($path => (
+        at => 3, %options
+    ));
+
+    # If you have less items than the index, then it's the same as
+    # as add_route -- it's just appended to the end of the list
+    $router->insert_route($path => (
+        at => 1_000_000, %options
+    ));
+
+    # If you want to prepend, omit "at", or specify 0
+    $router->insert_Route($path => (
+        at => 0, %options
+    ));
+
 =item B<routes>
 
 =item B<match ($path)>
@@ -353,10 +379,9 @@ if no routes match.
 This is still a relatively new module, even though it has been 
 sitting on my drive un-used for over a year now. We are only just 
 now using it at $work, so there still may be bugs lurking. For that
-very reason I have made the C<$DEBUG> variable more accessible 
-so that you can turn on the verbose debug logging with:
-
-  $Path::Router::DEBUG = 1;
+very reason I have made the C<DEBUG> flag more accessible 
+so that you can turn on the verbose debug logging with the 
+C<PATH_ROUTER_DEBUG> environment variable.
 
 And possibly help clear out some bugs lurking in the dark corners
 of this module. 
@@ -373,7 +398,7 @@ Stevan Little E<lt>stevan.little@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2008 Infinity Interactive, Inc.
+Copyright 2008-2009 Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
